/**
 * 
 */
package com.atlassian.crucible.plugin.export.servlet;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Locale;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import com.atlassian.crucible.plugin.export.CrucibleRestClient;
import com.atlassian.crucible.plugin.export.ReviewExporter;
import com.atlassian.crucible.plugin.export.exception.ExportException;
import com.atlassian.crucible.plugin.export.exception.ValidationException;
import com.atlassian.crucible.spi.services.ReviewService;
import com.cenqua.fisheye.config.SpringContext;

/**
 * Servlet that provides access to the Review Export functionality.
 * 
 * @author Ross Rowe
 * 
 */
public class ExportServlet extends HttpServlet {

	private static final String INPUT_HTML = "/download/resources/com.atlassian.crucible.plugin.export/input.html";

	private static final String CONTENT_TYPE = "application/pdf";

	private static final long serialVersionUID = 1896736152949054896L;

	private static final Logger logger = Logger.getLogger(ExportServlet.class);

	private static final String USER_NAME = "username";

	private static final String PASSWORD = "password";

	private static final String REVIEW_ID = "reviewId";

	/**
	 * Writes the generated PDF to the servlet output stream.
	 * 
	 * @param response
	 * @param pdfOutputStream
	 * @throws IOException
	 */
	private void writePDFResponse(HttpServletResponse response,
			ByteArrayOutputStream pdfOutputStream) throws IOException {
		response.setHeader("Expires", "0");
		response.setHeader("Cache-Control",
				"must-revalidate, post-check=0, pre-check=0");
		response.setHeader("Content-disposition",
				"attachment; filename=export.pdf");
		response.setHeader("Pragma", "public");
		response.setContentType(CONTENT_TYPE);
		response.setContentLength(pdfOutputStream.size());
		ServletOutputStream out = response.getOutputStream();
		pdfOutputStream.writeTo(out);
		out.flush();
	}

	/**
	 * Display the input html file upon a GET request.
	 */
	@Override
	protected void doGet(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		RequestDispatcher dispatcher = request.getRequestDispatcher(INPUT_HTML);
		dispatcher.forward(request, response);
	}

	/**
	 * Processes a HTTP POST request by validating the mandatory fields and
	 * generating the PDF report.
	 */
	@Override
	protected void doPost(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		validateParameters(request);
		generateReport(request, response);
	}

	/**
	 * Validates that the userName, password and reviewId form variables are not
	 * blank.
	 * 
	 * @param request
	 * @throws ValidationException
	 *             if an input field is null or blank
	 */
	private void validateParameters(HttpServletRequest request)
			throws ValidationException {
		String userName = request.getParameter(USER_NAME);
		String password = request.getParameter(PASSWORD);
		String reviewId = request.getParameter(REVIEW_ID);
		if (StringUtils.isBlank(userName))
			throw new ValidationException("User name must be specified");
		if (StringUtils.isBlank(password))
			throw new ValidationException("Password must be specified");
		if (StringUtils.isBlank(reviewId))
			throw new ValidationException("Review Id must be specified");
	}

	/**
	 * Invokes the {@link ReviewExporter} class to produce the PDF export of the
	 * Crucible review.
	 * 
	 * @param request
	 * @param response
	 * @throws ServletException
	 * @throws IOException
	 */
	private void generateReport(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		//Use the Rest client instead of the direct client until a fix for CRUC-291 is released
//		ReviewService rs = (ReviewService) SpringContext
//				.getComponent("reviewService");
		// CrucibleClient client = new CrucibleDirectClient(rs);
		CrucibleRestClient client = new CrucibleRestClient();
		ReviewExporter reviewExporter = new ReviewExporter(client, Locale.getDefault());
		String baseUrl = request.getScheme() + "://" + request.getServerName()
				+ ":" + request.getServerPort();
		reviewExporter.setBaseUrl(baseUrl);
		client.setBaseUrl(baseUrl);
		String userName = request.getParameter(USER_NAME);
		String password = request.getParameter(PASSWORD);
		String reviewId = request.getParameter(REVIEW_ID);
		ByteArrayOutputStream pdfOutputStream = null;
		try {
			pdfOutputStream = reviewExporter.createPdfExport(userName,
					password, reviewId);
		} catch (ExportException e) {
			logger.error(e);
			throw new ServletException(e);
		}
		writePDFResponse(response, pdfOutputStream);
	}
}
